﻿using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Text;
using System.Runtime.Serialization.Formatters.Binary;
using System.Windows.Forms;

using TestClient.DataService;

namespace TestClient
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.getDataButton.Click += getDataButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 데이터 구하기 버튼 클릭시 처리하기 - downloadButton_Click(sender, e)

        /// <summary>
        /// 데이터 구하기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void getDataButton_Click(object sender, EventArgs e)
        {
            int dataCount;

            try
            {
                dataCount = Convert.ToInt32(this.dataCountTextBox.Text);
            }
            catch
            {
                MessageBox.Show("유효한 데이터 카운트를 입력해 주시기 바랍니다.");

                return;
            }

            this.textBox.Text = string.Empty;

            WriteMessage("START GetData : DataCount={0}", dataCount);

            try
            {
                Enabled = false;

                DateTime startTime = DateTime.Now;

                using(DataServiceClient client = new DataServiceClient("NetTcpBinding_IDataService"))
                {
                    List<string> list;

                    if(this.useGZipStreamCheckBox.Checked)
                    {
                        Stream stream = client.GetData(dataCount, true);
                    
                        GZipStream gZipStream = new GZipStream(stream, CompressionMode.Decompress);
                    
                        BinaryFormatter binaryFormatter = new BinaryFormatter();
                    
                        list = (List<string>)binaryFormatter.Deserialize(gZipStream);
                    
                        gZipStream.Close();
                    
                        stream.Close();
                    }
                    else
                    {
                        Stream stream = client.GetData(dataCount, false);
                        
                        BinaryFormatter binaryFormatter = new BinaryFormatter();
                        
                        list = (List<string>)binaryFormatter.Deserialize(stream);
                        
                        stream.Close();
                    }

                    StringBuilder stringBuilder = new StringBuilder();

                    for(int i = 0; i < list.Count; i++)
                    {
                        stringBuilder.AppendLine(list[i]);
                    }

                    this.textBox.Text = stringBuilder.ToString().Trim();
                }

                DateTime endTime = DateTime.Now;

                WriteMessage("END GetData : Elapsed={0}", (endTime - startTime).ToString());
            }
            finally
            {
                Enabled = true;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 메시지 지우기 - ClearMessage()

        /// <summary>
        /// 메시지 지우기
        /// </summary>
        private void ClearMessage()
        {
            this.messageLabel.Text = string.Empty;
        }

        #endregion
        #region 메시지 쓰기 - WriteMessage(format, parameterArray)

        /// <summary>
        /// 메시지 쓰기
        /// </summary>
        /// <param name="format">포맷 문자열</param>
        /// <param name="parameterArray">매개 변수 배열</param>
        private void WriteMessage(string format, params object[] parameterArray)
        {
            string message;

            if(parameterArray.Length == 0)
            {
                message = format;
            }
            else
            {
                message = string.Format(format, parameterArray);
            }

            messageLabel.Text = string.Format("[{0}] {1}", DateTime.Now.ToString("HH:mm:ss"), message);
        }

        #endregion
    }
}