﻿using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Runtime.Serialization.Formatters.Binary;

namespace TestServer
{
    /// <summary>
    /// 데이터 서비스
    /// </summary>
    public class DataService : IDataService
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 데이터 구하기 - GetData(dataCount, useGZipStream)

        /// <summary>
        /// 데이터 구하기
        /// </summary>
        /// <param name="dataCount">데이터 카운트</param>
        /// <param name="useGZipStream">GZipStream 사용 여부</param>
        /// <returns>스트림</returns>
        public Stream GetData(int dataCount, bool useGZipStream)
        {
            WriteMessage("START GetData : DataCount={0}", dataCount);

            #region 결과 리스트를 생성한다.

            List<string> resultlist = new List<string>();

            for(int i = 0; i < dataCount; i++)
            {
                resultlist.Add(Guid.NewGuid().ToString().ToUpper());
            }

            #endregion

            if(useGZipStream)
            {
                BinaryFormatter binaryFormatter = new BinaryFormatter();
            
                MemoryStream memoryStream = new MemoryStream();
            
                GZipStream gZipStream = new GZipStream(memoryStream, CompressionMode.Compress, true);
            
                binaryFormatter.Serialize(gZipStream, resultlist);
            
                gZipStream.Close();
            
                memoryStream.Position = 0;
            
                WriteMessage("END GetData");
            
                return memoryStream;
            }
            else
            {
                BinaryFormatter binaryFormatter = new BinaryFormatter();
            
                MemoryStream memoryStream = new MemoryStream();
            
                binaryFormatter.Serialize(memoryStream, resultlist);
            
                memoryStream.Position = 0;
            
                WriteMessage("END GetData");
            
                return memoryStream;
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 메시지 쓰기 - WriteMessage(format, parameterArray)

        /// <summary>
        /// 메시지 쓰기
        /// </summary>
        /// <param name="format">포맷 문자열</param>
        /// <param name="parameterArray">매개 변수 배열</param>
        private void WriteMessage(string format, params object[] parameterArray)
        {
            string message;

            if(parameterArray.Length == 0)
            {
                message = format;
            }
            else
            {
                message = string.Format(format, parameterArray);
            }

            Console.WriteLine(string.Format("[{0}] {1}", DateTime.Now.ToString("HH:mm:ss"), message));
        }

        #endregion
    }
}